﻿''' <summary>
''' DragObject  : the image being dragged
''' </summary>
Public Class DragObject
    Implements IDisposable

#Region " Variables "

    Private m_image As Bitmap
    Private m_cursor As Cursor

#End Region

#Region " Constructor "

    Public Sub New(ByVal image As Bitmap)
        m_image = image
        CreateCursor()  'when an image is dragged, you can see it in the cursor
    End Sub

#End Region

#Region " Dispose "
    Private disposedValue As Boolean = False    ' To detect redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' dispose managed state (managed objects).
                If m_image IsNot Nothing Then m_image.Dispose()
                If m_cursor IsNot Nothing Then m_cursor.Dispose()
            End If
            ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
            ' TODO: set large fields to null.
        End If
        Me.disposedValue = True
    End Sub

#Region " IDisposable Support "

    ' TODO: override Finalize() only if Dispose(ByVal disposing As Boolean) above has code to free unmanaged resources.
    'Protected Overrides Sub Finalize()
    '    ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
    '    Dispose(False)
    '    MyBase.Finalize()
    'End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

#End Region

#Region " Public "

#Region " Events "
    Public Event GiveFeedback(ByVal e As GiveFeedbackEventArgs)
#End Region

#Region " Properties "

    ''' <summary>
    ''' The image being dragged.
    ''' </summary>
    Public Property Image() As Bitmap
        Get
            Return m_image
        End Get
        Set(ByVal value As Bitmap)
            m_image = value
        End Set
    End Property

    ''' <summary>
    ''' A cursor used during the drag comprising of the image being dragged with an arrow
    ''' cursor superimposed. 
    ''' </summary> 
    Public ReadOnly Property ImageCursor() As Cursor
        Get
            Return m_cursor
        End Get
    End Property

#End Region

#Region " Methods "
    Public Sub RaiseGiveFeedback(ByVal sender As Object, ByVal e As GiveFeedbackEventArgs)
        RaiseEvent GiveFeedback(e)
    End Sub
#End Region

#End Region

#Region " Private "

#Region " Methods "

#Region " CreateCursor "

    Private Sub CreateCursor()

        If m_cursor IsNot Nothing Then m_cursor.Dispose()

        ' Create a cursor from the image.
        ' Resolution of the image shouldn't effect the size.
        ' Draw the image into a specific rectangle of a bitmap.
        Dim cursorImage As New Bitmap(m_image.Width, m_image.Height)
        Dim g As Graphics = Graphics.FromImage(cursorImage)
        g.DrawImage(m_image, New Rectangle(0, 0, cursorImage.Width, cursorImage.Height))
        m_cursor = New Cursor(cursorImage.GetHicon)
        g.Dispose()
        cursorImage.Dispose()

    End Sub

#End Region

#End Region

#End Region

End Class

