﻿''' <summary> 
''' The image that was dropped onto a DropTargetControl.
''' TODO: resize image method
''' </summary>
Public Class DroppedImage
    Implements IDisposable

#Region " Variables "
    Private m_image As Bitmap
    Private m_rotatedImage As Bitmap
    Private m_orientation As Single
    Private m_location As Point
    Private m_boundingRectangle As Rectangle
    Private m_pointClicked As Point
    ' number of steps in a complete circle.  We rotate 1 step at a time.
    Private m_rotationSteps As Single
    Private m_selected As Boolean
#End Region

#Region " Constructor "
    Public Sub New(ByVal image As Bitmap, ByVal location As Point)
        m_image = image
        m_rotatedImage = image.Clone
        m_location = location
        m_boundingRectangle = New Rectangle(m_location.X, m_location.Y, m_image.Width, m_image.Height)
        m_orientation = 0
        m_rotationSteps = 36
    End Sub
#End Region

#Region " Dispose "

    Private disposedValue As Boolean = False    ' To detect redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' dispose managed state (managed objects).
                If m_image IsNot Nothing Then m_image.Dispose()
                If m_rotatedImage IsNot Nothing Then m_rotatedImage.Dispose()
            End If

            ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
            ' TODO: set large fields to null.
        End If
        Me.disposedValue = True
    End Sub

    ' TODO: override Finalize() only if Dispose(ByVal disposing As Boolean) above has code to free unmanaged resources.
    'Protected Overrides Sub Finalize()
    '    ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
    '    Dispose(False)
    '    MyBase.Finalize()
    'End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region

#Region " Public "

#Region " Properties "

    ''' <summary>
    ''' The rectangle specifying the location and size of the image in the DropTargetControl.
    ''' </summary>
    Public ReadOnly Property BoundingRectangle() As Rectangle
        Get
            Return m_boundingRectangle
        End Get
    End Property

    ''' <summary>
    ''' The unrotated image.
    ''' </summary> 
    Public Property Image() As Bitmap
        Get
            Return m_image
        End Get
        Set(ByVal value As Bitmap)
            m_image = value
        End Set
    End Property
    ''' <summary>
    ''' The location of the top left corner of the image in the DropTargetControl.
    ''' </summary>
    Public Property Location() As Point
        Get
            Return m_location
        End Get
        Set(ByVal value As Point)
            m_location = value
            m_boundingRectangle = New Rectangle(m_location.X, m_location.Y, m_image.Width, m_image.Height)
        End Set
    End Property
    ''' <summary>
    ''' Then the user clicks the image in the dropTargetControl, this is set to the point clicked
    ''' in the image co-ordinates.
    ''' Note: Used to set the hotspot in the cursor if the click is the start of a drag.
    ''' </summary>
    Public Property PointClicked() As Point
        Get
            Return m_pointClicked
        End Get
        Set(ByVal value As Point)
            m_pointClicked = value
        End Set
    End Property

    ''' <summary>
    ''' The rotated version of the image property.
    ''' </summary> 
    Public ReadOnly Property RotatedImage() As Bitmap
        Get
            Return m_rotatedImage
        End Get
    End Property

    Public Property Selected() As Boolean
        Get
            Return m_selected
        End Get
        Set(ByVal value As Boolean)
            m_selected = value
        End Set
    End Property

#End Region

#Region " Methods "
    ''' <summary>
    ''' Rotate the image in the clockwise direction.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub RotateClockwise()
        Rotate(True)
    End Sub

    ''' <summary>
    ''' Rotate the image in the anticlockwise direction.
    ''' </summary>
    Public Sub RotateCounterClockwise()
        Rotate(False)
    End Sub

#End Region

#End Region

#Region " Private "

#Region " Methods "

    Private Sub Rotate(ByVal clockwise As Boolean)
        Dim g As Graphics = Graphics.FromImage(m_rotatedImage)
        g.Clear(Color.Transparent)
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        ' set the new angle
        If clockwise Then
            m_orientation += 360 / (m_rotationSteps)
        Else
            m_orientation -= 360 / (m_rotationSteps)
        End If
        ' cap the values
        If m_orientation > 360 Then
            m_orientation = m_orientation - 360
        ElseIf m_orientation < 0 Then
            m_orientation = 360 + m_orientation
        End If
        ' move the origin of the axis the graphics is using to the center point of the bitmap
        g.TranslateTransform(m_image.Width / 2, m_image.Height / 2)
        ' rotate the axis about the new origin
        g.RotateTransform(m_orientation)
        ' draw the original image
        g.DrawImageUnscaled(m_image, New Point(-(m_image.Width / 2), -(m_image.Height / 2)))
        g.Dispose()
    End Sub

#End Region

#End Region

End Class
